import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { MapOverlay, OverlayGroup, PageOf } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { ThunkConfig } from '@/types/store';
import { getError } from '@/utils/error-report/getError';
import { pageableSchema } from '@/models/pageableSchema';
import { overlayGroupSchema } from '@/models/overlayGroupSchema';
import { showToast } from '@/utils/showToast';
import axios from 'axios';
import { apiPath } from '../apiPath';

export const getOverlayGroups = createAsyncThunk<OverlayGroup[], void, ThunkConfig>(
  'overlayGroup/getOverlayGroups',
  async () => {
    try {
      const response = await axiosInstanceNewAPI.get<PageOf<OverlayGroup>>(
        apiPath.getOverlayGroups(),
        {
          withCredentials: true,
        },
      );

      const isDataValid = validateDataUsingZodSchema(
        response.data,
        pageableSchema(overlayGroupSchema),
      );

      return isDataValid ? response.data.content : [];
    } catch (error) {
      return Promise.reject(getError({ error, prefix: 'Failed to fetch overlay groups' }));
    }
  },
);

type AddOverlayGroupArgs = {
  name: string;
  order: number;
};

export const addOverlayGroup = createAsyncThunk<undefined, AddOverlayGroupArgs, ThunkConfig>(
  'overlays/addOverlayGroup',
  async (
    { name, order },
    { dispatch },
    // eslint-disable-next-line consistent-return
  ) => {
    try {
      const response = await axiosInstanceNewAPI.post<MapOverlay>(
        apiPath.addOverlayGroup(),
        {
          name,
          order,
        },
        {
          withCredentials: true,
        },
      );

      const isDataValid = validateDataUsingZodSchema(response.data, overlayGroupSchema);
      if (!isDataValid) {
        showToast({
          type: 'error',
          message: 'Problem with adding group encountered',
          duration: 120000,
        });
      } else {
        showToast({ type: 'success', message: 'Overlay group added successfully' });
      }
      await dispatch(getOverlayGroups());
    } catch (error) {
      if (axios.isAxiosError(error) && error.code === 'ERR_BAD_REQUEST') {
        const data = error.response?.data;
        showToast({ type: 'error', message: data.reason, duration: 120000 });
      } else {
        return Promise.reject(getError({ error, prefix: 'Failed to add overlay group' }));
      }
    }
  },
);
